/* PROGRAM NAME : ESTGAM1.PRG*/
/* SMOOTH TRANSITION REGRESSION FOR KOREA */
/* GAUSS-NEWTON NONLINEAR LEAST SQUARES ESTIMATION WITH LEADS-AND-LAGS */
/* GAMMA IS ESTIMATED BY NLLS AND GAUSS-NEWTON METHOD. */
/* 5 parameters */

LIBRARY  OPTMUM;
#INCLUDE OPTMUM.EXT;

format 18,4;
output file = c:\gauss\gaussprc\str\quarter\k\co_80_01.out reset;

/* sample range is 1970:1 to 1999:3 */
load av[184,6] = c:\gauss\gaussprc\str\data\quarter.prn;
av   = av[100:184,.];
iip  = av[.,5];
m1   = av[.,2];
m2   = av[.,1];
cr   = av[.,6];
cpi  = av[.,3];

/*  setting dependet and independent variables   */
lrm1 = ln(m1)-ln(cpi);
lrm2 = ln(m2)-ln(cpi);
liip = ln(iip);

Y  = lrm1-liip;
X2 = cr;

T  = ROWS(Y);


@ PROCEDURE THAT COMPUTES THE FUNCTION TO BE MINIMIZED  @

PROC (1) = OBJF(GC);

LOCAL GF,ZZ,OB;

GF = 1-1/( 1+EXP(-GC[1]*( X2-GC[2] ) ) );
ZZ = ONES(T,1)~X2~(X2.*GF);
OB = Y'Y - Y'ZZ*INV(ZZ'ZZ)*ZZ'Y;
RETP(OB/T);
ENDP;

IINI    = ZEROS(2,1);
IINI[1] = 0.5;
IINI[2] = MEDIAN(X2);

OPTSET;
_OPALGR  = 6;
__OUTPUT = 0;
_OPGTOL  = 0.001;
_OPMITER = 10000;
{GC,FV,GR,COV} = OPTMUM(&OBJF,IINI);

GFH       = 1-1/( 1+EXP(-GC[1]*( X2-GC[2] ) ) );
Z         = ONES(T,1)~X2~(X2.*GFH);
IHAT      = ZEROS(5,1);
IHAT[1:3] = INV(Z'Z)*Z'Y;
IHAT[4:5] = GC;

lag=1;
{S1K1,STD11,B11U,U11U,B11E1,E11E1,B11E2,E11E2} = GNLL(X2,Y,IHAT,lag);
{P11U}  = P(U11U);
{P11E1} = P(E11E1);
{P11E2} = P(E11E2);

{S2K1,STD21,B21U,U21U,B21E1,E21E1,B21E2,E21E2} = GNLL(X2,Y,S1K1,lag);
{P21U}  = P(U21U);
{P21E1} = P(E21E1);
{P21E2} = P(E21E2);

lag=2;
{S1K2,STD12,B12U,U12U,B12E1,E12E1,B12E2,E12E2} = GNLL(X2,Y,IHAT,lag);
{P12U}  = P(U12U);
{P12E1} = P(E12E1);
{P12E2} = P(E12E2);

{S2K2,STD22,B22U,U22U,B22E1,E22E1,B22E2,E22E2} = GNLL(X2,Y,S1K1,lag);
{P22U}  = P(U22U);
{P22E1} = P(E22E1);
{P22E2} = P(E22E2);


lag=4;
{S1K4,STD14,B14U,U14U,B14E1,E14E1,B14E2,E14E2} = GNLL(X2,Y,IHAT,lag);
{P14U}  = P(U14U);
{P14E1} = P(E14E1);
{P14E2} = P(E14E2);

{S2K4,STD24,B24U,U24U,B24E1,E24E1,B24E2,E24E2} = GNLL(X2,Y,S1K1,lag);
{P24U}  = P(U24U);
{P24E1} = P(E24E1);
{P24E2} = P(E24E2);

"Algorithm";;_OPALGR;
"NLLS";
 IHAT';

"1-STEP, K=1";
 S1K1';
 STD11';
 "U1";; B11U;;U11U;;P11U;
 "E1";; B11E1;;E11E1;;P11E1;
 "E2";; B11E2;;E11E2;;P11E2;

"";

"2-STEP, K=1";
 S2K1';
 STD21';
 "U1";; B21U;;U21U;;P21U;
 "E1";; B21E1;;E21E1;;P21E1;
 "E2";; B21E2;;E21E2;;P21E2;

"";

"1-STEP, K=2";
 S1K2';
 STD12';
 "U1";; B12U;;U12U;;P12U;
 "E1";; B12E1;;E12E1;;P12E1;
 "E2";; B12E2;;E12E2;;P12E2;

"";

"2-STEP, K=2";
 S2K2';
 STD22';
 "U1";; B22U;;U22U;;P22U;
 "E1";; B22E1;;E22E1;;P22E1;
 "E2";; B22E2;;E22E2;;P22E2;

"";

"1-STEP, K=4";
 S1K4';
 STD14';
 "U1";; B14U;;U14U;;P14U;
 "E1";; B14E1;;E14E1;;P14E1;
 "E2";; B14E2;;E14E2;;P14E2;

"";

"2-STEP, K=4";
 S2K4';
 STD24';
 "U1";; B24U;;U24U;;P24U;
 "E1";; B24E1;;E24E1;;P24E1;
 "E2";; B24E2;;E24E2;;P24E2;

"";

end;



PROC (8) = GNLL(X2,Y,IHAT,K);

 @ X1 IS A MATRIX AND; X2 AND Y ARE VECTORS @
 @ Returns estimates for coefficients, gamma and c @

LOCAL Z1, J, Z2, Z, GH, GH1, GH2, GH3, U, U1, E1, E2, T, N, DGT1, DGT2, P1, P2, p21, p22, P3, P, QE, GNE, IPP,
      COE, A, B, LQ, LVAR, ST, X, R, CLL10, BLOCKU, BLOCKE1, BLOCKE2, Bmin, Bbig, Bstd, MINBU, MINBE1,
      MINBE2, COINU, COINE1, COINE2, m, Bstat;

T = ROWS(Y);
N = T-2*K-1;
X = X2;
R = COLS(X);

Z1 = X[2*K+2:T,.];
J  = 0;
DO UNTIL J==2*K;
J  = J + 1;
Z1 = Z1~X[2*K+2-J:T-J,.];
ENDO;

Z2 = X[2*K+1:T-1,.];
J  = 0;
DO UNTIL J==2*K;
J  = J + 1;
Z2 = Z2~X[2*K+1-J:T-1-J,.];
ENDO;

Z   = Z1-Z2;
GH  = 1-1/( 1+EXP(-IHAT[4]*( X2[K+2:T-K]-IHAT[5] ) ) );
GH1 = 1-1/( 1+EXP(-IHAT[4]*( X2[1:T]-IHAT[5] ) ) );
U   = Y[K+2:T-K] - IHAT[1]*ONES(T-2*K-1,1) - IHAT[2]*X2[K+2:T-K] - IHAT[3]*(X2[K+2:T-K].*GH);
U1  = Y[1:T] - IHAT[1]*ONES(T,1) - IHAT[2]*X2[1:T] - IHAT[3]*(X2[1:T].*GH1);

DGT1 = -(-(GH.*GH).*(EXP(-IHAT[4]*( X2[K+2:T-K]-IHAT[5] ) )).*(-X2[K+2:T-K]+IHAT[5] ));      @ Derivative w.r.t. gamma @
DGT2 = -(-(GH.*GH).*(EXP(-IHAT[4]*( X2[K+2:T-K]-IHAT[5] ) )).*ihat[4]);                      @ Derivative w.r.t. c @

P21 = (IHAT[3]*X2[K+2:T-K]).*DGT1;
P22 = (IHAT[3]*X2[K+2:T-K]).*DGT2;
p2  = p21~p22;
P3  = ONES(T-2*K-1,1)~X2[K+2:T-K]~(X2[K+2:T-K].*GH);
P   = P3~P2~Z;
QE  = IHAT|ZEROS(R*(2*K+1),1);
IPP = INV(P'P);
GNE = QE + IPP*P'U;

GH2 = 1-1/( 1+EXP(-GNE[4]*( X2[K+2:T-K]-GNE[5] ) ) );
E1  = Y[K+2:T-K] - GNE[1]*ONES(T-2*K-1,1) - GNE[2]*X2[K+2:T-K] - GNE[3]*(X2[K+2:T-K].*GH2);

GH3 = 1-1/( 1+EXP(-GNE[4]*( X2[K+2:T-K]-GNE[5] ) ) );
E2  = Y[K+2:T-K] - GNE[1]*ONES(T-2*K-1,1) - GNE[2]*X2[K+2:T-K] - GNE[3]*(X2[K+2:T-K].*GH3) - Z*GNE[6:(6+2*K)];

Bmin   = 10;
Bbig   = rows(U1)-4;
Bstat  = zeros(Bbig-Bmin+1,1);
for i(Bmin,Bbig,1);
{BLOCKU} = COINT(U1,i);
Bstat[i-Bmin+1,1] = BLOCKU;
endfor;

m     = 2;
Bstd  = zeros(Bbig-Bmin+1-2*m,1);
for i(1+m,Bbig-Bmin+1-m,1);
Bstd[i-m,1] = stdc(Bstat[i-m:i+m,1]);    
endfor;
MINBU = minindc(Bstd)+Bmin-1;

{COINU} = COINT(U1,MINBU);


Bmin = 10;
Bbig = rows(E1)-4;
Bstd = zeros(Bbig-Bmin+1,1);
for i(Bmin,Bbig,1);
{BLOCKE1} = COINT(E1,i);
Bstat[i-Bmin+1,1] = BLOCKE1;
endfor;

m     = 2;
Bstd  = zeros(Bbig-Bmin+1-2*m,1);
for i(1+m,Bbig-Bmin+1-m,1);
Bstd[i-m,1] = stdc(Bstat[i-m:i+m,1]);    
endfor;
MINBE1 = minindc(Bstd)+Bmin-1;

{COINE1} = COINT(E1,MINBE1);


Bmin = 10;
Bbig = rows(E2)-4;
Bstd = zeros(Bbig-Bmin+1,1);
for i(Bmin,Bbig,1);
{BLOCKE2} = COINT(E2,i);
Bstd[i-Bmin+1,1] = BLOCKE2;
endfor;

m     = 2;
Bstd  = zeros(Bbig-Bmin+1-2*m,1);
for i(1+m,Bbig-Bmin+1-m,1);
Bstd[i-m,1] = stdc(Bstat[i-m:i+m,1]);    
endfor;
MINBE2 = minindc(Bstd)+Bmin-1;

{COINE2} = COINT(E2,MINBE2);


{COE,A,B} = AR4(U);
LQ = ABEL4Q(T,COE);
LVAR = LVEQS(U,LQ);

ST = SQRT(DIAG(IPP)*LVAR);

   RETP(GNE[1:5],ST[1:5],MINBU,COINU,MINBE1,COINE1,MINBE2,COINE2);

ENDP;



proc (1) = P(z);
local N, SEQ, FAC, i, U, xl, y, GAM, CUM, PV, U0, xl0, y0;

N   = 10;
SEQ = seqa(1,1,N);
FAC = cumprodc(SEQ);

GAM = zeros(N+1,1);

for i(1,N,1);
U          = ((2^(1/2)/2)+2*i*(2^(1/2)))/(2*z^(1/2));
xl         =  U|0;
y          =  intquad1(&f,xl);
GAM[i+1,1] = (gamma(i+1/2)/FAC[i]*gamma(1/2))*((-1)^(i))*(1-(2/pi^(1/2))*y);
endfor;

U0         =  (2^(1/2)/2)/(2*z^(1/2));
xl0        =  U0|0;
y0         =  intquad1(&f,xl0);
GAM[1,1]   =  1-(2/pi^(1/2))*y0;

CUM      =  cumsumc(GAM);
PV       =  (2^(1/2))*CUM[N+1];

RETP(PV);
ENDP;

proc  f(x);
retp( exp(-(x.^2)) );
endp;



PROC (1) = COINT(U,BLO);
LOCAL MU, T, M, MB, i, COE, A, B, LQ, LVAR, UE, CNLLS, CNLLM;

   MU   = U;
   T    = rows(MU);
   M    = ceil(T/BLO);
   MB   = zeros(BLO,M);

   for i(1,M-1,1);
   MB[1:BLO,i] = MU[1+(i-1)*BLO:i*BLO];
   endfor;
   MB[1:BLO,M] = MU[T-BLO+1:T];

   CNLLS   = zeros(M,1);
   for i(1,M,1);
   {COE,A,B}  = AR4(MB[1:BLO,i]);
   LQ   = ABEL4Q(BLO,COE);
   LVAR = LVEQS(MB[1:BLO,i],LQ);
   UE   = BLO^(-2)*sumc((cumsumc(MB[1:BLO,i]))^(2));
   CNLLS[i,1] = LVAR^(-1)*UE;
   endfor;
   CNLLM = maxc(CNLLS);

RETP(CNLLM);

ENDP;



/*  ABEL4Q.G  */
/* AUTOMATIC BAND WIDTH ESTIMATOR WITH AN AR(4) APPROXIMATION */
/* QS WINDOW */
      PROC (1) = ABEL4Q(T,C);
      LOCAL  ALPHA;

   ALPHA = 4.*((C[1]*C[2] + 4.*C[1]*C[3] + 9.*C[1]*C[4] + C[2]*C[3]
    + 4.*C[2]*C[4] + C[3]*C[4] - C[1] - 4.*C[2] - 9.*C[3] - 16.*C[4])^2)
    /((C[1] + C[2] + C[3] + C[4] - 1.)^4);
   ALPHA = 1.3221*((ALPHA*T)^.2);

   RETP(ALPHA);
   ENDP;


/* ar4.g*/

    /* This program estiamtes ar(4) processes. */

proc(3) = ar4(x);

local t,y,y1,y2,y3,y4,i,z,izz,est,res,var,vcm;

t  = rows(x) - 4;
y  = zeros(t,1);
y1 = zeros(t,1);
y2 = zeros(t,1);
y3 = zeros(t,1);
y4 = zeros(t,1);
izz = zeros(4,4);

i = 0;
do until i==t;
i = i + 1;
    y[i] = x[i+4];
    y1[i] = x[i+3];
    y2[i] = x[i+2];
    y3[i] = x[i+1];
    y4[i] = x[i];
endo;

z = y1~y2~y3~y4;
izz = inv(z'z);
est = izz*z'y;
res = y - z*est;
var = res'res/(t-4);
vcm = var*izz;

retp(est,vcm,res);
endp;


/* THIS SUBROUTINE CALCULATES THE LONG VARIANCE OF A TIME SERIES USING
   THE QS WINDOW.*/

      PROC  LVEQS(U,L);
         LOCAL LVAR,N,JS,KR,SUM,WS,LP,KJ,X;


      N = ROWS(U);
      LVAR = 0;
      LVAR = U'U/N + LVAR;

      JS = 0;
      DO UNTIL JS== n-1;
      JS = JS + 1;
         SUM = 0.;
         KR = 0;
         LP = N-JS;
         DO UNTIL KR== LP;
         KR = KR + 1;
         KJ = KR+JS;
         SUM = SUM + U[KJ]*U[KR];
         ENDO;

         X=JS/L;
         WS = (25/(12*PI*PI*X*X))*( SIN(6*PI*X/5)/(6*PI*X/5) - COS(6*PI*X/5) );

      LVAR = LVAR + 2.*SUM*WS/N;
      ENDO;

      RETP(LVAR);
      ENDP;
















