/* PROGRAM NAME : ESTGAM1.PRG*/
/* SMOOTH TRANSITION REGRESSION FOR KOREA */
/* GAUSS-NEWTON NONLINEAR LEAST SQUARES ESTIMATION WITH LEADS-AND-LAGS */
/* GAMMA IS ESTIMATED BY NLLS AND GAUSS-NEWTON METHOD. */
/* 5 parameters */

LIBRARY  OPTMUM;
#INCLUDE OPTMUM.EXT;

format 18,4;
output file = c:\gauss\gaussprc\str\quarter\85_99_k.out reset;

/* sample range is 1970:1 to 1999:3 */
load av[184,6] = c:\gauss\gaussprc\str\data\quarter.prn;
av   = av[120:176,.];
iip  = av[.,5];
m1   = av[.,2];
m2   = av[.,1];
cr   = av[.,6];
cpi  = av[.,3];

/*  setting dependet and independent variables   */
lrm1 = ln(m1)-ln(cpi);
lrm2 = ln(m2)-ln(cpi);
liip = ln(iip);

Y  = lrm1-liip;
X2 = cr;

T  = ROWS(Y);


@ PROCEDURE THAT COMPUTES THE FUNCTION TO BE MINIMIZED  @

PROC (1) = OBJF(GC);

LOCAL GF,ZZ,OB;

GF = 1-1/( 1+EXP(-GC[1]*( X2-GC[2] ) ) );
ZZ = ONES(T,1)~X2~(X2.*GF);
OB = Y'Y - Y'ZZ*INV(ZZ'ZZ)*ZZ'Y;
RETP(OB/T);
ENDP;

IINI    = ZEROS(2,1);
IINI[1] = 0.5;
IINI[2] = MEDIAN(X2);

OPTSET;
_OPALGR  = 6;
__OUTPUT  = 0;
_OPGTOL  = 0.001;
_OPMITER = 10000;
{GC,FV,GR,COV} = OPTMUM(&OBJF,IINI);

GFH       = 1-1/( 1+EXP(-GC[1]*( X2-GC[2] ) ) );
Z         = ONES(T,1)~X2~(X2.*GFH);
IHAT      = ZEROS(5,1);
IHAT[1:3] = INV(Z'Z)*Z'Y;
IHAT[4:5] = GC;

lag=1;
{GNNLS1K1,STD11} = GNLL(X2,Y,IHAT,lag);

{GNNLS2K1,STD21} = GNLL(X2,Y,GNNLS1K1,lag);

lag=2;
{GNNLS1K2,STD12} = GNLL(X2,Y,IHAT,lag);

{GNNLS2K2,STD22} = GNLL(X2,Y,GNNLS1K2,lag);

lag=4;
{GNNLS1K3,STD13} = GNLL(X2,Y,IHAT,lag);

{GNNLS2K3,STD23} = GNLL(X2,Y,GNNLS1K3,lag);

lag=8;
{GNNLS1K4,STD14} = GNLL(X2,Y,IHAT,lag);

{GNNLS2K4,STD24} = GNLL(X2,Y,GNNLS1K4,lag);

e  = 0.975;
d  = T-6;
y  = cdftci(e,d);

CI11_h = GNNLS1K1-std11.*y[1];
CI11_l = GNNLS1K1+std11.*y[1];

CI21_h = GNNLS2K1-std21.*y[1];
CI21_l = GNNLS2K1+std21.*y[1];

CI12_h = GNNLS1K2-std12.*y[1];
CI12_l = GNNLS1K2+std12.*y[1];

CI22_h = GNNLS2K2-std22.*y[1];
CI22_l = GNNLS2K2+std22.*y[1];

CI13_h = GNNLS1K3-std13.*y[1];
CI13_l = GNNLS1K3+std13.*y[1];

CI23_h = GNNLS2K3-std23.*y[1];
CI23_l = GNNLS2K3+std23.*y[1];

CI14_h = GNNLS1K4-std14.*y[1];
CI14_l = GNNLS1K4+std14.*y[1];

CI24_h = GNNLS2K4-std24.*y[1];
CI24_l = GNNLS2K4+std24.*y[1];

"Algorothm";;_OPALGR ;
"NLLS";;IHAT';

" 1-STEP, K=1";
 GNNLS1K1';
 STD11';
"Upper CI";
 print CI11_h';
"Lower CI";
 print CI11_l';
"";

" 2-STEP, K=1";
 GNNLS2K1';
 STD21';
"Upper CI";
 print CI21_h';
"Lower CI";
 print CI21_l';
"";

" 1-STEP, K=2";
 GNNLS1K2';
 STD12';
"Upper CI";
 print CI12_h';
"Lower CI";
 print CI12_l';
"";

" 2-STEP, K=2";
 GNNLS2K2';
 STD22';
"Upper CI";
 print CI22_h';
"Lower CI";
 print CI22_l';
"";

" 1-STEP, K=4";
 GNNLS1K3';
 STD13';
"Upper CI";
 print CI13_h';
"Lower CI";
 print CI13_l';
"";

" 2-STEP, K=4";
 GNNLS2K3';
 STD23';
"Upper CI";
 print CI23_h';
"Lower CI";
 print CI23_l';
"";

" 1-STEP, K=8";
 GNNLS1K4';
 STD14';
"Upper CI";
 print CI14_h';
"Lower CI";
 print CI14_l';
"";

" 2-STEP, K=8";
 GNNLS2K4';
 STD24';
"Upper CI";
 print CI24_h';
"Lower CI";
 print CI24_l';
"";

end;

PROC(2) = GNLL(X2,Y,IHAT,K);

 @ X1 IS A MATRIX AND; X2 AND Y ARE VECTORS @
 @ Returns estimates for coefficients, gamma and c @

LOCAL Z1, J, Z2, Z, GH, U, T, DGT1, DGT2, P1, P2, p21, p22, P3, P, QE, GNE, IPP,
          COE, A, B, LQ, LVAR, ST, X, R;

T = ROWS(Y);
X = X2;
R = COLS(X);

Z1 = X[2*K+2:T,.];
J = 0;
DO UNTIL J==2*K;
J = J + 1;
Z1 = Z1~X[2*K+2-J:T-J,.];
ENDO;

Z2 = X[2*K+1:T-1,.];
J = 0;
DO UNTIL J==2*K;
J = J + 1;
Z2 = Z2~X[2*K+1-J:T-1-J,.];
ENDO;

Z = Z1-Z2;
GH =1-1/( 1+EXP(-IHAT[4]*( X2[K+2:T-K]-IHAT[5] ) ) );
U = Y[K+2:T-K] - IHAT[1]*ONES(T-2*K-1,1) - IHAT[2]*X2[K+2:T-K] - IHAT[3]*(X2[K+2:T-K].*GH);

DGT1 = -(-(GH.*GH).*(EXP(-IHAT[4]*( X2[K+2:T-K]-IHAT[5] ) )).*(-X2[K+2:T-K]+IHAT[5] ));      @ Derivative w.r.t. gamma @
DGT2 = -(-(GH.*GH).*(EXP(-IHAT[4]*( X2[K+2:T-K]-IHAT[5] ) )).*ihat[4]);                      @ Derivative w.r.t. c @

P21 = (IHAT[3]*X2[K+2:T-K]).*DGT1;
P22 = (IHAT[3]*X2[K+2:T-K]).*DGT2;
p2  = p21~p22;
P3  = ONES(T-2*K-1,1)~X2[K+2:T-K]~(X2[K+2:T-K].*GH);
P   = P3~P2~Z;
QE =  IHAT|ZEROS(R*(2*K+1),1);
IPP = INV(P'P);
GNE = QE + IPP*P'U;

{COE,A,B} = AR4(U);
LQ = ABEL4Q(T,COE);
LVAR = LVEQS(U,LQ);

ST = SQRT(DIAG(IPP)*LVAR);

   RETP(GNE[1:5],ST[1:5]);

ENDP;

/*  ABEL4Q.G  */
/* AUTOMATIC BAND WIDTH ESTIMATOR WITH AN AR(4) APPROXIMATION */
/* QS WINDOW */
      PROC (1) = ABEL4Q(T,C);
      LOCAL  ALPHA;

   ALPHA = 4.*((C[1]*C[2] + 4.*C[1]*C[3] + 9.*C[1]*C[4] + C[2]*C[3]
    + 4.*C[2]*C[4] + C[3]*C[4] - C[1] - 4.*C[2] - 9.*C[3] - 16.*C[4])^2)
    /((C[1] + C[2] + C[3] + C[4] - 1.)^4);
   ALPHA = 1.3221*((ALPHA*T)^.2);

   RETP(ALPHA);
   ENDP;


/* ar4.g*/

    /* This program estiamtes ar(4) processes. */

proc(3) = ar4(x);

local t,y,y1,y2,y3,y4,i,z,izz,est,res,var,vcm;


t  = rows(x) - 4;
y  = zeros(t,1);
y1 = zeros(t,1);
y2 = zeros(t,1);
y3 = zeros(t,1);
y4 = zeros(t,1);
izz = zeros(4,4);

i = 0;
do until i==t;
i = i + 1;
    y[i] = x[i+4];
    y1[i] = x[i+3];
    y2[i] = x[i+2];
    y3[i] = x[i+1];
    y4[i] = x[i];
endo;

z = y1~y2~y3~y4;
izz = inv(z'z);
est = izz*z'y;
res = y - z*est;
var = res'res/(t-4);
vcm = var*izz;



retp(est,vcm,res);
endp;


/* THIS SUBROUTINE CALCULATES THE LONG VARIANCE OF A TIME SERIES USING
   THE QS WINDOW.*/

      PROC  LVEQS(U,L);
         LOCAL LVAR,N,JS,KR,SUM,WS,LP,KJ,X;


      N = ROWS(U);
      LVAR = 0;
      LVAR = U'U/N + LVAR;

      JS = 0;
      DO UNTIL JS== n-1;
      JS = JS + 1;
         SUM = 0.;
         KR = 0;
         LP = N-JS;
         DO UNTIL KR== LP;
         KR = KR + 1;
         KJ = KR+JS;
         SUM = SUM + U[KJ]*U[KR];
         ENDO;

         X=JS/L;
         WS = (25/(12*PI*PI*X*X))*( SIN(6*PI*X/5)/(6*PI*X/5) - COS(6*PI*X/5) );

      LVAR = LVAR + 2.*SUM*WS/N;
      ENDO;

      RETP(LVAR);
      ENDP;














